package be.iminds.ilabt.jfed.espec.util;

import javafx.application.Platform;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

/**
 * LimitedLiveLogImpl
 */
public class LimitedLiveLogImpl implements LimitedLiveLog {
    private final List<ChangeListener> listeners = new ArrayList();

    @Nonnull private String head;
    @Nonnull private StringBuffer tail;

    private int received;
    private int limit;
    private int maxHeadLen;
    private int maxTailLen;
    private boolean endReached;

    public LimitedLiveLogImpl() {
        this(1_000_000);
    }
    public LimitedLiveLogImpl(int limit) {
        head = "";
        tail = new StringBuffer("");
        received = 0;
        this.limit = limit;
        this.maxHeadLen = limit/2;
        this.maxTailLen = limit - this.maxHeadLen;
        endReached = false;
    }

    @Override
    public void addChangeListener(@Nonnull ChangeListener l) {
        listeners.add(l);
    }

    @Override
    public void removeChangeListener(@Nonnull ChangeListener l) {
        listeners.remove(l);
    }

    @Override
    public void appendText(@Nonnull String extraText) {
        received += extraText.length();
        if (tail.length() == 0) {
            if (head.length() + extraText.length() > limit) {
                String fullText = head + extraText;

                head = fullText.substring(0, maxHeadLen);
                tail.append(fullText.substring(maxHeadLen));

                if (tail.length() > maxTailLen)
                    tail.delete(0, tail.length() - maxTailLen);
            } else {
                head += extraText;
            }
        } else {
            tail.append(extraText);
            if (tail.length() > maxTailLen)
                tail.delete(0, tail.length() - maxTailLen);
        }
        fireChange(extraText);
    }

    private void fireChange(@Nonnull String extraText) {
        for (ChangeListener l : listeners)
            l.onChanged(this, extraText);
    }

    @Override
    public void setEndReached() {
        this.endReached = true;
        Platform.runLater(() -> {
                    for (ChangeListener l : listeners)
                        l.onEndReached(this);
                }
        );
    }

    @Nonnull
    @Override
    public String getText() {
        if (tail.length() == 0)
            return head;
        else
            return head + "\n...\n\n" + tail.toString();
    }

    @Nonnull
    @Override
    public String getHeadText() {
        return head;
    }

    @Nonnull
    @Override
    public String getTailText() {
        return tail.toString();
    }

    @Override
    public boolean isLimited() {
        return tail.length() > 0;
    }

    @Override
    public int getReceivedSize() {
        return received;
    }

    @Override
    public int getStoredSize() {
        return head.length() + tail.length();
    }

    @Override
    public int getSizeLimit() {
        return limit;
    }

    @Override
    public boolean isEndReached() {
        return endReached;
    }
}
